==> kbibtex-0.1.3.tar.bz2: kbibtex-0.1.3/src/bibtexfileexporterbibtex.cpp <==
/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <bibtexfile.h>
#include <bibtexelement.h>
#include <bibtexentry.h>
#include <bibtexstring.h>
#include <bibtexcomment.h>
#include <encoderlatex.h>

#include "bibtexfileexporterbibtex.h"

namespace BibTeX
{

    BibTeXFileExporterBibTeX::BibTeXFileExporterBibTeX() : BibTeXFileExporter()
    {
        cancelFlag = FALSE;
        m_stringOpenDelimiter = '"';
        m_stringCloseDelimiter = '"';
    }


    BibTeXFileExporterBibTeX::~BibTeXFileExporterBibTeX()
    {
        // nothing
    }


    bool BibTeXFileExporterBibTeX::save( QIODevice* iodevice, BibTeXFile* bibtexfile )
    {
        bool result = TRUE;
        QTextStream stream( iodevice );

        writePreamble( stream, bibtexfile->preamble() );

        // first, write strings at the beginning
        for ( BibTeXFile::ElementList::iterator it = bibtexfile->elements.begin(); it != bibtexfile->elements.end() && result && !cancelFlag; it++ )
        {
            BibTeXString *string = dynamic_cast<BibTeXString*>( *it );
            if ( string != NULL )
                result &= writeString( stream, string );
        }

        // second, write cross-referencing elements
        for ( BibTeXFile::ElementList::iterator it = bibtexfile->elements.begin(); it != bibtexfile->elements.end() && result && !cancelFlag; it++ )
        {
            BibTeXEntry *entry = dynamic_cast<BibTeXEntry*>( *it );
            if ( ( entry != NULL ) && ( entry->getField( BibTeXEntryField::ftCrossRef ) != NULL ) )
                result &= writeEntry( stream, entry );
        }

        // third, write remaining elements
        for ( BibTeXFile::ElementList::iterator it = bibtexfile->elements.begin(); it != bibtexfile->elements.end() && result && !cancelFlag; it++ )
        {
            BibTeXEntry *entry = dynamic_cast<BibTeXEntry*>( *it );
            if ( entry != NULL )
            {
                if ( entry->getField( BibTeXEntryField::ftCrossRef ) == NULL )
                    result &= writeEntry( stream, entry );
            }
            else
            {
                BibTeXComment *comment = dynamic_cast<BibTeXComment*>( *it );
                if ( comment != NULL )
                    result &= writeComment( stream, comment );
            }
        }

        return result && !cancelFlag;
    }

    bool BibTeXFileExporterBibTeX::save( QIODevice* iodevice, BibTeXElement* element )
    {
        bool result = FALSE;
        QTextStream stream( iodevice );
        BibTeXEntry *entry = dynamic_cast<BibTeXEntry*>( element );
        if ( entry != NULL )
            result |= writeEntry( stream, entry );
        else
        {
            BibTeXString * string = dynamic_cast<BibTeXString*>( element );
            if ( string != NULL )
                result |= writeString( stream, string );
            else
            {
                BibTeXComment * comment = dynamic_cast<BibTeXComment*>( element );
                if ( comment != NULL )
                    result |= writeComment( stream, comment );
            }
        }

        return result;
    }

    void BibTeXFileExporterBibTeX::cancel()
    {
        cancelFlag = TRUE;
    }

    bool BibTeXFileExporterBibTeX::writeEntry( QTextStream&stream, BibTeXEntry* entry )
    {
        stream << "@" << entry->entryTypeString() << "{ " << entry->id();

        for ( BibTeXEntry::EntryFields::iterator it = entry->begin(); it != entry->end(); it++ )
        {
            BibTeXEntryField *field = *it;
            stream << ',' << endl << '\t' << field->fieldTypeName() << " = " << itemToString( field->begin(), field->end(), field->fieldType() );
        }
        stream << endl << "}" << endl << endl;
        return TRUE;
    }

    bool BibTeXFileExporterBibTeX::writeString( QTextStream&stream, BibTeXString *string )
    {
        stream << "@string{ " << string->key() << " = " << itemToString( string->begin(), string->end() ) << " }" << endl << endl;

        return TRUE;
    }

    bool BibTeXFileExporterBibTeX::writeComment( QTextStream&stream, BibTeXComment *comment )
    {
        if ( comment->usePercent() )
        {
            QStringList commentLines = QStringList::split( '\n', EncoderLaTeX::currentEncoderLaTeX() ->encode( comment->text() ) );
            for ( QStringList::Iterator it = commentLines.begin(); it != commentLines.end(); it++ )
            {
                //                 qDebug( "writeComment %s", ( *it ).latin1() );
                stream << '%' << *it << endl;
            }
            stream << endl;
        }
        else
        {
            stream << "@comment{" << EncoderLaTeX::currentEncoderLaTeX() ->encode( comment->text() ) << "}" << endl << endl;
        }
        return TRUE;
    }

    bool BibTeXFileExporterBibTeX::writePreamble( QTextStream &stream, const QString& preamble )
    {
        if ( !preamble.isEmpty() )
            stream << "@preamble{" << EncoderLaTeX::currentEncoderLaTeX() ->encode( preamble ) << "}" << endl << endl;
        return TRUE;
    }

    void BibTeXFileExporterBibTeX::setStringDelimiter( const QChar& stringOpenDelimiter, const QChar& stringCloseDelimiter )
    {
        m_stringOpenDelimiter = stringOpenDelimiter;
        m_stringCloseDelimiter = stringCloseDelimiter;
    }

    QString BibTeXFileExporterBibTeX::itemToString( const BibTeXElement::FieldItems::iterator& begin, const BibTeXElement::FieldItems::iterator& end, const BibTeXEntryField::FieldType fieldType )
    {
        QString result;
        bool inString = FALSE;
        bool isFirst = TRUE;
        EncoderLaTeX *encoder = EncoderLaTeX::currentEncoderLaTeX();

        for ( BibTeXElement::FieldItems::const_iterator it = begin; it != end; it++ )
        {
            BibTeXElement::FieldItem fieldItem = *it;
            if ( !inString && !fieldItem.isStringKey )
            {
                if ( !isFirst )
                    result.append( " # " );
                result.append( m_stringOpenDelimiter );
                inString = TRUE;
            }
            else if ( inString && fieldItem.isStringKey )
            {
                result.append( m_stringCloseDelimiter );
                if ( !isFirst )
                    result.append( " # " );
                inString = FALSE;
            }

            if ( fieldItem.isStringKey )
                result.append( fieldItem.text );
            else
            {
                result.append( encoder->encodeSpecialized( fieldItem.text, fieldType ) );
            }

            isFirst = FALSE;
        }

        if ( inString )
            result.append( m_stringCloseDelimiter );
        return result;
    }

}

==> kbibtex-0.1.4.tar.bz2: kbibtex-0.1.4/src/bibtexfileexporterbibtex.cpp <==
/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <bibtexfile.h>
#include <bibtexelement.h>
#include <bibtexentry.h>
#include <bibtexstring.h>
#include <bibtexcomment.h>
#include <encoderlatex.h>

#include "bibtexfileexporterbibtex.h"

namespace BibTeX
{

    BibTeXFileExporterBibTeX::BibTeXFileExporterBibTeX() : BibTeXFileExporter(),
            m_stringOpenDelimiter( '"' ), m_stringCloseDelimiter( '"' ), m_encoding( BibTeXFile::encLaTeX ), cancelFlag( FALSE )
    {
        // nothing
    }


    BibTeXFileExporterBibTeX::~BibTeXFileExporterBibTeX()
    {
        // nothing
    }


    bool BibTeXFileExporterBibTeX::save( QIODevice* iodevice, BibTeXFile* bibtexfile, QStringList * /*errorLog*/ )
    {
        bool result = TRUE;
        QTextStream stream( iodevice );
        if ( m_encoding == BibTeXFile::encUTF8 )
            stream.setEncoding( QTextStream::UnicodeUTF8 );

        writePreamble( stream, bibtexfile->preamble() );

        // first, write strings at the beginning
        for ( BibTeXFile::ElementList::iterator it = bibtexfile->elements.begin(); it != bibtexfile->elements.end() && result && !cancelFlag; it++ )
        {
            BibTeXString *string = dynamic_cast<BibTeXString*>( *it );
            if ( string != NULL )
                result &= writeString( stream, string );
        }

        // second, write cross-referencing elements
        for ( BibTeXFile::ElementList::iterator it = bibtexfile->elements.begin(); it != bibtexfile->elements.end() && result && !cancelFlag; it++ )
        {
            BibTeXEntry *entry = dynamic_cast<BibTeXEntry*>( *it );
            if ( ( entry != NULL ) && ( entry->getField( BibTeXEntryField::ftCrossRef ) != NULL ) )
                result &= writeEntry( stream, entry );
        }

        // third, write remaining elements
        for ( BibTeXFile::ElementList::iterator it = bibtexfile->elements.begin(); it != bibtexfile->elements.end() && result && !cancelFlag; it++ )
        {
            BibTeXEntry *entry = dynamic_cast<BibTeXEntry*>( *it );
            if ( entry != NULL )
            {
                if ( entry->getField( BibTeXEntryField::ftCrossRef ) == NULL )
                    result &= writeEntry( stream, entry );
            }
            else
            {
                BibTeXComment *comment = dynamic_cast<BibTeXComment*>( *it );
                if ( comment != NULL )
                    result &= writeComment( stream, comment );
            }
        }

        return result && !cancelFlag;
    }

    bool BibTeXFileExporterBibTeX::save( QIODevice* iodevice, BibTeXElement* element, QStringList * /*errorLog*/ )
    {
        bool result = FALSE;
        QTextStream stream( iodevice );
        if ( m_encoding == BibTeXFile::encUTF8 )
            stream.setEncoding( QTextStream::UnicodeUTF8 );

        BibTeXEntry *entry = dynamic_cast<BibTeXEntry*>( element );
        if ( entry != NULL )
            result |= writeEntry( stream, entry );
        else
        {
            BibTeXString * string = dynamic_cast<BibTeXString*>( element );
            if ( string != NULL )
                result |= writeString( stream, string );
            else
            {
                BibTeXComment * comment = dynamic_cast<BibTeXComment*>( element );
                if ( comment != NULL )
                    result |= writeComment( stream, comment );
            }
        }

        return result;
    }

    void BibTeXFileExporterBibTeX::cancel()
    {
        cancelFlag = TRUE;
    }

    bool BibTeXFileExporterBibTeX::writeEntry( QTextStream&stream, BibTeXEntry* entry )
    {
        stream << "@" << entry->entryTypeString() << "{ " << entry->id();

        for ( BibTeXEntry::EntryFields::const_iterator it = entry->begin(); it != entry->end(); it++ )
        {
            BibTeXEntryField *field = *it;
            stream << ',' << endl << '\t' << field->fieldTypeName() << " = " << valueToString( field->value(), field->fieldType() );
        }
        stream << endl << "}" << endl << endl;
        return TRUE;
    }

    bool BibTeXFileExporterBibTeX::writeString( QTextStream&stream, BibTeXString *string )
    {
        stream << "@string{ " << string->key() << " = " << valueToString( string->value() ) << " }" << endl << endl;

        return TRUE;
    }

    bool BibTeXFileExporterBibTeX::writeComment( QTextStream&stream, BibTeXComment *comment )
    {
        if ( comment->usePercent() )
        {
            QString text = comment->text() ;
            switch ( m_encoding )
            {
            case BibTeXFile::encLaTeX:
                text = EncoderLaTeX::currentEncoderLaTeX() ->encode( text );
                break;
            default:
                stream.setEncoding( QTextStream::UnicodeUTF8 );
            }
            QStringList commentLines = QStringList::split( '\n', text );
            for ( QStringList::Iterator it = commentLines.begin(); it != commentLines.end(); it++ )
            {
                //                 qDebug( "writeComment %s", ( *it ).latin1() );
                stream << '%' << *it << endl;
            }
            stream << endl;
        }
        else
        {
            QString text = comment->text() ;
            switch ( m_encoding )
            {
            case BibTeXFile::encLaTeX:
                text = EncoderLaTeX::currentEncoderLaTeX() ->encode( text );
                break;
            default:
                stream.setEncoding( QTextStream::UnicodeUTF8 );
            }
            stream << "@comment{" << text << "}" << endl << endl;
        }
        return TRUE;
    }

    bool BibTeXFileExporterBibTeX::writePreamble( QTextStream &stream, const QString& preamble )
    {
        if ( !preamble.isEmpty() )
        {
            QString text = preamble ;
            switch ( m_encoding )
            {
            case BibTeXFile::encLaTeX:
                text = EncoderLaTeX::currentEncoderLaTeX() ->encode( text );
                break;
            default:
                stream.setEncoding( QTextStream::UnicodeUTF8 );
            }

            stream << "@preamble{" << text << "}" << endl << endl;
        }
        return TRUE;
    }

    void BibTeXFileExporterBibTeX::setStringDelimiter( const QChar& stringOpenDelimiter, const QChar& stringCloseDelimiter )
    {
        m_stringOpenDelimiter = stringOpenDelimiter;
        m_stringCloseDelimiter = stringCloseDelimiter;
    }

    void BibTeXFileExporterBibTeX::setEncoding( BibTeXFile::Encoding encoding )
    {
        m_encoding = encoding;
    }

    QString BibTeXFileExporterBibTeX::valueToString( const BibTeXValue *value, const BibTeXEntryField::FieldType fieldType )
    {
        QString result;
        bool isFirst = TRUE;
        EncoderLaTeX *encoder = EncoderLaTeX::currentEncoderLaTeX();

        for ( BibTeXValue::ValueItems::const_iterator it = value->begin(); it != value->end(); it++ )
        {
            if ( !isFirst )
                result.append( " # " );
            isFirst = FALSE;

            BibTeXValueItem *valueItem = *it;
            if ( !valueItem->isStringKey() )
            {
                QString text = valueItem->text() ;
                switch ( m_encoding )
                {
                case BibTeXFile::encLaTeX:
                    text = encoder->encodeSpecialized( text, fieldType ) ;
                    break;
                default:
                    {
                        // nothing
                    }
                }

                result.append( m_stringOpenDelimiter );
                result.append( text );
                result.append( m_stringCloseDelimiter );
            }
            else
                result.append( valueItem->text() );

            //             qDebug( "valueToString=%s", result.latin1() );
        }

        return result;
    }

}
